<?php
declare(strict_types = 1);

/**
 * Méthodes spécifiques à l'application.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
class App
{
	/**
	 * Nom du script courant ('admin' ou 'gallery').
	 *
	 * @var string
	 */
	public static $scriptName = 'gallery';

	/**
	 * Chemin d'accès à la page courante.
	 *
	 * @var string
	 */
	public static $scriptPath = '/';



	/**
	 * Vérifie si un nom d'utilisateur, un mot ou une adresse IP
	 * se trouve dans une des listes noires correspondantes.
	 *
	 * @param string $form
	 * @param string $name
	 * @param string $email
	 * @param string $message
	 * @param array $post
	 *
	 * @return array|bool
	 */
	public static function blacklists(string $form = '',
	string $name = '', string $email = '', string $message = '', array $post = [])
	{
		$check = function(string $str, string $list, bool $word = FALSE)
		{
			$list = preg_split('`[\r\n]+`', $list, -1, PREG_SPLIT_NO_EMPTY);

			foreach ($list as &$entry)
			{
				$ent = preg_quote($entry);
				$ent = str_replace(['\\*', '\\?'], ['[^\s]*', '.'], $ent);
				$ent = Utility::removeAccents($ent);

				$regex = $word ? '(?:^|\W)(' . $ent . ')(?:$|\W)' : '^(' . $ent . ')$';
				if (preg_match('`' . $regex . '`ui', $str, $m))
				{
					return ['entry' => $entry, 'match' => $m[1]];
				}
			}

			return TRUE;
		};

		// Vérification de l'adresse IP.
		if (($r = $check((string) $_SERVER['REMOTE_ADDR'],
		Config::$params['blacklist_ips'])) !== TRUE)
		{
			$rejected =
			[
				'entry' => $r['entry'],
				'list' => 'ips',
				'text' => __('Votre adresse IP est bannie.')
			];
		}

		// Vérification du nom d'utilisateur.
		if ($name && ($r = $check(Utility::removeAccents($name),
		Config::$params['blacklist_names'])) !== TRUE)
		{
			$rejected =
			[
				'entry' => $r['entry'],
				'list' => 'names',
				'text' => __('Ce nom d\'utilisateur est banni.')
			];
		}

		// Vérification de l'adresse de courriel.
		if ($email && ($r = $check(Utility::removeAccents($email),
		Config::$params['blacklist_emails'])) !== TRUE)
		{
			$rejected =
			[
				'entry' => $r['entry'],
				'list' => 'emails',
				'text' => __('Cette adresse de courriel est bannie.')
			];
		}

		// Vérification du message.
		if ($message && ($r = $check(Utility::removeAccents($message),
		Config::$params['blacklist_words'], TRUE)) !== TRUE)
		{
			$rejected =
			[
				'entry' => $r['entry'],
				'list' => 'words',
				'text' => sprintf(
					__('Votre message contient un mot non autorisé : "%s".'), $r['match']
				)
			];
		}

		if (isset($rejected))
		{
			// Log d'activité.
			if ($form)
			{
				self::logActivity(
					$form . '_rejected_blacklist_' . $rejected['list'], $rejected['entry'], $post
				);
			}

			return $rejected;
		}

		return TRUE;
	}

	/**
	 * Définit proprement les valeurs des cases à cocher d'un formulaire.
	 *
	 * @return void
	 */
	public static function checkboxes(&$arr)
	{
		foreach ($arr as $k => &$v)
		{
			if (!is_array($v))
			{
				continue;
			}
			if ($k == '_checkboxes')
			{
				foreach ($v as &$name)
				{
					$arr[$name] = (int) array_key_exists($name, $arr);
				}
			}
			else
			{
				self::checkboxes($v);
			}
		}
	}

	/**
	 * Retourne une clé de sécurité pour protéger l'accès aux fichiers.
	 *
	 * @param array $params
	 *   Paramètres identifiants de façon unique le fichier.
	 * @param string $date
	 *   Date, pour limiter la durée de validité de la clé.
	 *
	 * @return string
	 */
	public static function fileKeyHash(array $params, string $date = ''): string
	{
		if (!CONF_FILE_KEY)
		{
			return md5('This file is in open access.');
		}

		return md5(implode('|', array_merge($params, [
			Auth::getCookieSessionToken(),
			CONF_ACCESS_KEY,
			$date ? $date : date('Y-m-d')
		])));
	}

	/**
	 * Vérifie que les paramètres $params correspondent à la clé $key.
	 *
	 * @param array $params
	 *   Paramètres identifiants de façon unique le fichier.
	 * @param string $key
	 *   Clé à vérifier.
	 *
	 * @return bool
	 */
	public static function fileKeyVerify(array $params, string $key): bool
	{
		if (!CONF_FILE_KEY)
		{
			return TRUE;
		}

		return self::fileKeyHash($params) == $key
			|| self::fileKeyHash($params, date('Y-m-d', time() - 7200)) == $key;
	}

	/**
	 * Création des paramètres GET liés aux filtres.
	 *
	 * @return void
	 */
	public static function filtersGET(): void
	{
		// Filtres sans paramètre.
		$filters =
		[
			'comments',
			'favorites',
			'images',
			'items',
			'pending',
			'recent-images',
			'recent-items',
			'recent-videos',
			'views',
			'videos',
			'votes'
		];

		// Filtres avec paramètre.
		$filters_p =
		[
			'action',
			'camera-brand',
			'camera-model',
			'date',
			'date-added',
			'date-created',
			'ip',
			'item',
			'note',
			'search',
			'tag',
			'result',
			'user',
			'user-favorites',
			'user-images',
			'user-items',
			'user-videos'
		];

		$regexp = implode('|', array_map('preg_quote', array_merge($filters, $filters_p)));
		$regexp = "`/(?:($regexp)(?:/([^/]+))?(?:/([^/]+))?)$`i";
		if (preg_match($regexp, $_GET['q_pageless'], $m))
		{
			// Filtre sur une catégorie.
			if ($_GET['section'] == 'category')
			{
				$_GET['section_file'] = 'album';
				$_GET['album_id'] = $_GET['category_id'];
				
			}

			// Nom du filtre.
			$_GET['filter'] = $m[1];

			// Type de filtre.
			$_GET['filter_type'] = in_array($_GET['filter'], $filters) ? 'filter' : 'filter_p';

			// Valeur du filtre.
			$_GET['filter_value'] = $m[2] ?? '';

			// Catégorie dans laquelle s'applique le filtre.
			$_GET['filter_cat_id'] = $m[3] ?? $_GET['category_id'] ?? 0;
			if ($_GET['section'] == 'item' && count($m) == 3)
			{
				$_GET['filter_cat_id'] = $m[2];
			}

			// Partie de la requête correspondant au filtre.
			$_GET['q_filter'] = $m[0];

			// Requête sans filtre.
			$_GET['q_filterless'] = preg_replace($regexp, '', $_GET['q_pageless']);
		}
	}

	/**
	 * Formate une durée depuis un temps en secondes.
	 *
	 * @param mixed $seconds
	 *
	 * @return string
	 */
	public static function formatDuration($seconds): string
	{
		$t = round((int) $seconds);
		return $seconds > 3599
			? sprintf('%02d:%02d:%02d', ($t/3600), ($t/60%60), $t%60)
			: sprintf('%02d:%02d', ($t/60%60), $t%60);
	}

	/**
	 * Retourne l'URL d'un fichier.
	 *
	 * @param string $path
	 *   Chemin d'accès au fichier.
	 * @param bool $path
	 *   Télécharger le fichier ?
	 *
	 * @return string
	 */
	public static function getFileSource(string $path, bool $download = FALSE): string
	{
		$key = self::fileKeyHash([$path]);
		$file = $download ? 'download' : 'file';

		return CONF_GALLERY_PATH . "/$file.php?key=$key&file=$path";
	}

	/**
	 * Génère le chemin d'accès à l'application.
	 *
	 * @return string
	 */
	public static function getGalleryPath(): string
	{
		$gallery_path = $_SERVER['SCRIPT_NAME'];
		$gallery_path = preg_replace('`^[a-z\d]+://[^/]+/`i', '/', $gallery_path);
		$gallery_path = dirname(dirname($gallery_path));
		$gallery_path = str_replace(' ', '%20', $gallery_path);
		$gallery_path = (preg_match('`^[./]*$`', $gallery_path)) ? '' : $gallery_path;
		$gallery_path = preg_replace('`(?:[\x5c]|/+$)`', '', $gallery_path);

		return $gallery_path;
	}

	/**
	 * Retourne le chemin d'un fichier selon le type d'image redimensionnée demandé.
	 *
	 * @param string $type
	 *   Type de vignette.
	 * @param string $path
	 *   Chemin d'accès de l'élément.
	 * @param string $id
	 *   Identifiant de l'élément.
	 * @param string $adddt
	 *   Date d'ajout de l'élément.
	 * @param string $size
	 *   Dimensions max. de l'image.
	 * @param int $orientation
	 *   Orientation de l'image (valeur du paramètre EXIF).
	 * @param string $quality
	 *   Qualité GD de l'image.
	 * @param string $crop
	 *   Paramètres de rognage (uniquement pour vignettes).
	 *
	 * @return string
	 */
	public static function getImageFilepath(string $type, string $path, string $id,
	string $adddt, string $size, int $orientation, string $quality,
	string $crop = ''): string
	{
		if (!preg_match('`^\d{2,4}(x\d{2,4})?$`', $size)
		 || !preg_match('`^\d{2,3}$`', $quality))
		{
			return '';
		}

		switch ($type)
		{
			case 'cat' :
				$dir = 'thumbs';
				break;

			case 'item' :
			case 'pending' :
				$dir = 'thumbs';
				break;

			case 'resize' :
				$dir = 'resize';
				break;

			default :
				return '';
		}

		$ext = strtolower(preg_replace('`^.+(\.[a-z0-9]{2,4})$`i', '$1', $path));
		$ext = ($ext == '.mp4' || $ext == '.webm') ? '.jpg' : $ext;

		$params = $orientation > 1 && $orientation < 9
			? [$size, $quality, $adddt, $crop, $orientation]
			: [$size, $quality, $adddt, $crop];

		return 'cache/' . $dir . '/'
			. $id . '-'
			. self::hashFilename($id, $params)
			. $ext;
	}

	/**
	 * Retourne l'URL d'une image redimensionnée.
	 *
	 * @param string $path
	 *   Chemin d'accès au fichier.
	 *
	 * @return string
	 */
	public static function getImageResize(array $i): string
	{
		// Paramètres.
		$item = $i['item_path'];
		$quality = (int) Config::$params['items_resize_quality'];
		$height = (int) Config::$params['items_resize_height'];
		$width = (int) Config::$params['items_resize_width'];
		$size = $width . 'x' . $height;
		$orientation = (int) $i['item_orientation'];

		// Si l'image n'est pas plus grande que les dimensions maximum,
		// ou si les dimensions maximum sont égales à 0
		// on retourne l'URL du fichier original.
		if (($i['item_width'] <= $width && $i['item_height'] <= $height)
		|| ($width < 1 && $height < 1))
		{
			return self::getFileSource($i['item_path']);
		}

		// Nom de fichier de l'image redimensionnée.
		$file = self::getImageFilepath('resize', $item, $i['item_id'],
			$i['item_adddt'], $size, $orientation, (string) $quality);
		$file = basename($file);

		// Clé de sécurité.
		$params = $orientation > 1 && $orientation < 9
			? [$file, $item, $quality, $size, $orientation]
			: [$file, $item, $quality, $size];
		$key = self::fileKeyHash($params);

		return CONF_GALLERY_PATH . "/resize.php?"
			. "file=$file&item=$item&key=$key&quality=$quality&size=$size"
			. ($orientation > 1 && $orientation < 9 ? "&orientation=$orientation" : "");
	}

	/**
	 * Retourne la description d'une catégorie ou d'un fichier
	 * à destination de la balise <meta name="description">.
	 *
	 * @param string $desc
	 *
	 * @return string
	 */
	public static function getMetaDescription(string $desc): string
	{
		$desc = preg_split('`[\n\r]+`', $desc);

		return Utility::trimAll(strip_tags($desc[0]));
	}

	/**
	 * Retourne le chemin du répertoire système
	 * utilisé pour les fichiers temporaires.
	 *
	 * @return mixed
	 *   Retourne le chemin absolu du répertoire temporaire,
	 *   ou FALSE en cas d'erreur.
	 */
	public static function getTempDirSys()
	{
		if (function_exists('sys_get_temp_dir'))
		{
			return realpath(sys_get_temp_dir());
		}
		return FALSE;
	}

	/**
	 * Retourne l'emplacement d'une vignette.
	 *
	 * @param string $type
	 *   Type de vignette ('cat', 'item', 'pending').
	 * @param array $i
	 *   Informations utiles de l'élément.
	 * @param string $size
	 *   Dimensions de la vignette.
	 * @param string $quality
	 *   Qualité de l'image.
	 * @param bool $poster
	 *   Indique s'il s'agit d'une vignette de vidéo.
	 *
	 * @return string
	 */
	public static function getThumbSource(string $type,
	array $i, string $size = '', string $quality = '', bool $poster = FALSE): string
	{
		// Paramètres.
		if (!preg_match('`^\d{2,4}(x\d{2,4})?$`', $size))
		{
			$size = (string) ($type == 'cat' ? CONF_THUMBS_SIZE_CAT : CONF_THUMBS_SIZE_ITEM);
		}

		if (!preg_match('`^\d{2,3}$`', $quality))
		{
			$quality = (string) CONF_THUMBS_QUALITY;
		}

		$adddt = (string) $i['item_adddt'];
		$id = (string) $i['item_id'];
		$id = ($type == 'pending') ? "p$id" : $id;
		$item = (string) $i['item_path'];
		$orientation = (int) $i['item_orientation'];

		// Paramètres de rognage (uniquement pour la galerie).
		$crop = '';
		if (self::$scriptName == 'gallery' && $type != 'pending' && !$poster)
		{
			$crop = $type == 'cat' ? ($i['cat_tb_params'] ?? '') : ($i['item_tb_params'] ?? '');
			$test = str_replace(['{', '}', '"', 'x', 'y', 'w', 'h', ':'], '', $crop);
			$crop = preg_match('`^[\d,]{7,32}$`', $test) ? $test : '';
		}

		// Chemin de la vignette.
		$path = self::getImageFilepath($type, $item, $id,
			$adddt, $size, $orientation, $quality, $crop);

		// Vidéos.
		$no_thumb = '';
		if (Item::isVideo($i['item_type']))
		{
			$hash = Video::uniqueHash($id, $adddt);
			$capture_file = Video::getCapture((int) $i['item_id'], $adddt, $type == 'pending');
			$capture_file = GALLERY_ROOT . '/cache/captures/' . $capture_file;
			$item = $id . '-' . $hash . '.vid';
			if (!file_exists($capture_file))
			{
				$no_thumb = CONF_GALLERY_PATH . '/images/video-no-thumb.png#';
			}
		}

		// Si la vignette n'est pas sécurisée et qu'elle existe déjà,
		// on peut l'afficher directement.
		if (!CONF_THUMBS_SECURE && file_exists(GALLERY_ROOT . '/' . $path))
		{
			return CONF_GALLERY_PATH . '/' . $path;
		}

		$file = basename($path);
		$params = $orientation > 1 && $orientation < 9
			? [$file, $item, $quality, $size, $type, $crop, $orientation]
			: [$file, $item, $quality, $size, $type, $crop];
		$key = self::fileKeyHash($params);

		return $no_thumb . CONF_GALLERY_PATH . '/thumb.php?'
			. "file=$file&item=$item&key=$key&quality=$quality&size=$size"
			. ($orientation > 1 && $orientation < 9 ? "&orientation=$orientation" : "")
			. "&type=$type"
			. ($crop ? "&crop=$crop" : "");
	}

	/**
	 * Retourne le chemin du fichier $file_infos chargé par POST
	 * déplacé dans le répertoire $dir.
	 *
	 * @param array $file_infos
	 *   Informations du fichier dans la variable $_FILES.
	 *   Exemple : $_FILES['userfile']
	 * @param string $dir
	 *   Chemin absolu du répertoire où transférer le fichier.
	 *
	 * @return mixed
	 *   Retourne le chemin du fichier (string),
	 *   ou bien un tableau d'erreur (array).
	 */
	public static function getUploadedFilepath(array &$file_infos, string $dir)
	{
		// Gestion des erreurs d'envoi utilisateur.
		switch ($error = (int) $file_infos['error'])
		{
			// Aucune erreur.
			case 0 :
				break;

			// Fichier trop lourd.
			case 1 :
			case 2 :
				return
				[
					'error' => $error,
					'text' => __('Fichier trop lourd.')
				];

			// Aucun fichier.
			case 4 :
				return
				[
					'error' => $error,
					'text' => __('Aucun fichier envoyé.')
				];

			// Erreur inconnue.
			default :
				return
				[
					'error' => $error,
					'text' => __('Erreur inconnue.')
				];
		}

		// On déplace l'image vers le répertoire destination.
		$filename = App::getValidFilename($file_infos['name']);
		$filepath = $dir . '/' . $filename;
		if (!move_uploaded_file($file_infos['tmp_name'], $filepath))
		{
			return FALSE;
		}

		return $filepath;
	}

	/**
	 * Retourne le chemin d'accès à une page de l'application
	 * depuis la racine du site.
	 *
	 * @param string $page
	 * @param bool $p_only
	 *
	 * @return string
	 */
	public static function getURL(string $page = '', bool $p_only = FALSE): string
	{
		// Encodage.
		$page = str_replace(['%23', '%2F'], ['#', '/'], rawurlencode($page));

		// Valeur du paramètre d'URL seulement.
		if ($p_only)
		{
			return $page;
		}

		// Si URL rewriting ou aucune page, on ne met pas "?q=".
		$q = ((CONF_URL_REWRITE && self::$scriptPath == '/') || Utility::isEmpty($page))
			? ''
			: '?q=';

		return CONF_GALLERY_PATH . self::$scriptPath . $q . $page;
	}

	/**
	 * Génère un lien vers l'admin depuis la galerie.
	 *
	 * @return string
	 */
	public static function getURLAdmin(string $url = ''): string
	{
		$script_path = self::$scriptPath;
		self::$scriptPath = '/' . CONF_ADMIN_DIR . '/';
		$url = self::getURL($url);
		self::$scriptPath = $script_path;

		return $url;
	}

	/**
	 * Génère un lien vers la galerie depuis l'admin.
	 *
	 * @return string
	 */
	public static function getURLGallery(string $url = ''): string
	{
		$script_path = self::$scriptPath;
		self::$scriptPath = '/';
		$url = self::getURL($url);
		self::$scriptPath = $script_path;

		return $url;
	}

	/**
	 * Retourne le nom d'URL d'un objet.
	 *
	 * @param string $name
	 *   Nom de l'objet.
	 *
	 * @return string
	 */
	public static function getURLName(string $name): string
	{
		$name = str_replace('/', '-', $name);
		$name = preg_replace('`[^-\d\w_]`ui', ' ', $name);
		$name = Utility::trimAll($name);
		$name = preg_replace('`[-\s]+`', '-', $name);
		$name = Utility::removeAccents($name);
		$name = mb_strtolower($name);

		if ($name === '')
		{
			$name = '-';
		}

		return $name;
	}

	/**
	 * Retourne un nom de répertoire valide à partir de celui fourni.
	 *
	 * @param string $dirname
	 *   Nom de répertoire.
	 *
	 * @return string
	 */
	public static function getValidDirname(string $dirname): string
	{
		$dirname = Utility::UTF8($dirname);
		$dirname = Utility::removeAccents($dirname);
		$dirname = preg_replace('`([^-_a-z0-9])`ui', '_', $dirname);

		if (strlen($dirname) > 220)
		{
			$dirname = substr($dirname, 0, 220);
		}

		return $dirname;
	}

	/**
	 * Retourne un nom de fichier valide à partir de celui fourni.
	 *
	 * @param string $filename
	 *   Nom de fichier.
	 *
	 * @return string
	 */
	public static function getValidFilename(string $filename): string
	{
		$filename = Utility::UTF8($filename);
		$filename = Utility::removeAccents($filename);
		$filename = preg_replace('`([^-_a-z0-9.]|\.(?![^.]+$))`ui', '_', $filename);

		if (strlen($filename) > 220)
		{
			$parts = preg_split('`\.(?=[^.]+$)`', $filename);
			if ($parts[0] === '')
			{
				$filename = substr($filename, 0, 220);
			}
			else
			{
				$parts[0] = substr($parts[0], 0, 200);
				$parts[1] = substr($parts[1], 0, 20);
				$filename = $parts[0] . '.' . $parts[1];
			}
		}

		return $filename;
	}

	/**
	 * Hashage MD5 d'un nom de fichier.
	 *
	 * @param string $id
	 *   Identifiant ou nom de de fichier.
	 * @param array $params
	 *   Paramètres supplémentaire à ajouter à la signature.
	 *
	 * @return string
	 */
	public static function hashFilename(string $id, array $params = []): string
	{
		return md5((string) $id . '|' . CONF_KEY . '|' . implode('|', $params));
	}

	/**
	 * Envoi un code de réponse HTTP et retourne le message
	 * correspondant à ce code.
	 *
	 * @param int $code
	 *   Code de réponse HTTP.
	 *
	 * @return string
	 */
	public static function httpResponse(int $code): string
	{
		$line = CONF_DEBUG_MODE ? '[' . debug_backtrace()[0]['line'] . '] ' : '';

		switch ($code)
		{
			case 400 :
				header('HTTP/1.1 400 Bad Request');
				return $line . 'Bad request.';

			case 403 :
				header('HTTP/1.1 403 Forbidden');
				return $line . 'Access forbidden.';

			case 404 :
				header('HTTP/1.1 404 Not Found');
				return $line . 'Not found.';

			case 500 :
				header('HTTP/1.1 500 Internal Server Error');
				return $line . 'Internal Server Error.';
		}
	}

	/**
	 * Enregistre l'activité de l'utilisateur.
	 *
	 * @param string $action
	 *   Action effectuée par l'utilisateur.
	 * @param string $match
	 *   Élément qui a été trouvée par une liste noire.
	 * @param array $post
	 *   Données en POST.
	 *
	 * @return void
	 */
	public static function logActivity(string $action, string $match = '', array $post = []): void
	{
		if (!Config::$params['users_log_activity'])
		{
			return;
		}

		if (Config::$params['users_log_activity_rejected_only']
		&& !strstr($action, '_rejected'))
		{
			return;
		}

		if ($post)
		{
			foreach ($post as $k => &$v)
			{
				if (Utility::isEmpty((string) $v))
				{
					unset($post[$k]);
					continue;
				}
				Utility::strLimit((string) $v, 255);
			}
			$post = Utility::jsonEncode($post);
		}

		$sql = "INSERT INTO {users_logs}
			   (user_id, log_page, log_date, log_action,
			   log_match, log_post, log_ip)
			   VALUES
			   (:user_id, :page, NOW(), :action,
			   :match, :post, :ip)";
		$params =
		[
			'action' => $action,
			'ip' => $_SERVER['REMOTE_ADDR'],
			'match' => mb_substr($match, 0, 255),
			'page' => in_array('Ajax', get_declared_classes()) ? 'ajax' : ($_GET['q'] ?? ''),
			'post' => is_string($post) ? $post : NULL,
			'user_id' => Auth::$id
		];
		DB::execute($sql, $params);
	}

	/**
	 * Hashage d'un mot de passe.
	 *
	 * @param string $password
	 *
	 * @return string
	 */
	public static function passwordHash(string $password): string
	{
		return password_hash($password, PASSWORD_BCRYPT);
	}

	/**
	 * Vérification d'un mot de passe.
	 *
	 * @param string $password
	 * @param string $hash
	 *
	 * @return string
	 */
	public static function passwordVerify(string $password, string $hash): bool
	{
		return password_verify($password, $hash);
	}

	/**
	 * Redirige vers une autre page de l'application.
	 *
	 * @param string $page
	 *   Page vers laquelle rediriger.
	 * @param int $http_response_code
	 *   Code de réponse HTTP.
	 *
	 * @return void
	 */
	public static function redirect(string $page = '', int $http_response_code = 302)
	{
		// Permet de connaître toutes les redirections en mode débogage.
		if (CONF_DEBUG_MODE)
		{
			$trace = debug_backtrace();
			$file = (strpos($trace[0]['file'], GALLERY_ROOT) === 0)
				? substr($trace[0]['file'], strlen(GALLERY_ROOT) + 1)
				: $trace[0]['file'];
			trigger_error('Redirect in ' . $file . ':' . $trace[0]['line'], E_USER_NOTICE);
		}

		// Vérification du format de l'URL générée.
		$url = GALLERY_HOST . self::getURL($page);
		$url = str_replace('../', '', $url);
		if (!preg_match('`^' . Utility::URLRegexp() . '`i', $url))
		{
			trigger_error('No valid URL: ' . $url, E_USER_WARNING);
			return;
		}

		// Redirection.
		header('Location: ' . $url, TRUE, $http_response_code);
		die;
	}

	/**
	 * Extraction des paramètres de la requête GET à partir de modèles.
	 *
	 * Exemple :
	 *   Page demandée :
	 *      ?q=album/12/page/4
	 *   Modèle correspondant :
	 *      album/{id}
	 *   Regexp créée à partir du modèle qui sera testée sur le paramètre "q" :
	 *      ^album/(\d{1,11})(?:-[^/]{1,255})?(?:/page/(\d{1,11}))?$
	 *
	 * Le paramètre "page" est toujours ajouté à la fin du modèle pour
	 * éviter de devoir répéter ce paramètre pour de nombreux modèles.
	 *
	 * @param array $patterns
	 *
	 * @return void
	 */
	public static function request(array $patterns): void
	{
		$_GET['page'] = 1;

		if (!isset($_GET['q']))
		{
			return;
		}

		// Sécurité : supprime les caractères spéciaux sensibles.
		$_GET['q'] = str_replace(['<', '>', '&', '"', "'", "\\"], '?', $_GET['q']);
		$_GET['q'] = preg_replace("`[\s\t]+`", '_', $_GET['q']);
		$_GET['q'] = Utility::deleteInvisibleChars($_GET['q']);

		// On crée un tableau de tous les paramètres.
		$q = explode('/', $_GET['q']);

		// On parcours le tableau des modèles.
		foreach ($patterns as $i => &$pattern)
		{
			if (strstr($pattern, '{category}'))
			{
				$patterns[] = str_replace('{category}', 'album', $pattern);
				$patterns[] = str_replace('{category}', 'category', $pattern);
				unset($patterns[$i]);
			}
		}
		foreach ($patterns as &$pattern)
		{
			// Si la section principale ne correspond pas au modèle,
			// inutile d'aller plus loin.
			if (substr($pattern, 0, strlen($q[0])) != $q[0])
			{
				continue;
			}

			// On crée la regexp correspondant au modèle.
			$pattern = explode('/', $pattern . '/{page}');
			for ($i = 0, $params = [''], $regex = ''; $i < count($pattern); $i++)
			{
				switch ($pattern[$i])
				{
					case '{date}' :
						$params[] = 'date';
						$regex .= '/(\d{4}-\d{2}-\d{2})';
						break;

					case '{date-flex}' :
						$params[] = 'date';
						$regex .= '/(\d{4}(?:-\d{2}){0,2})';
						break;

					case '{id}' :
						$p = explode('-', $pattern[$i - 1]);
						if (!strstr($p[0], '{'))
						{
							$params[] = $p[0] . '_id';
						}
						$regex .= '/(\d{1,11})(?:-[^/]{1,255})?';
						break;

					case '{ip}' :
						$params[] = 'ip';
						$regex .= '/(' . Utility::URLRegexp('ip') . ')';
						break;

					case '{key}' :
						$params[] = 'key';
						$regex .= '/([\da-zA-Z]{8,40})';
						break;

					case '{page}' :
						$params[] = '';
						$params[] = 'page';
						$regex .= '(?:/(page)/([1-9]|\d{2,12}))?';
						break;

					case '{search}' :
						$params[] = 'search';
						$regex .= '/([\da-zA-Z]{12})';
						break;

					default :
						$params[] = '';
						$regex .= '/(' . str_replace('(', '(?:', $pattern[$i]) . ')';
				}
			}

			// On teste la regexp.
			if (preg_match('`^' . substr($regex, 1) . '$`', $_GET['q'], $m))
			{
				// Si ça correspond, on crée les paramètres GET
				// à partir des chaînes capturées par la regexp.
				for ($i = 0; $i < count($params); $i++)
				{
					if (strstr($params[$i], '(') && substr($params[$i], -3) == '_id')
					{
						if (preg_match('`' . substr($params[$i], 0, -3) . '`', $_GET['q'], $m2))
						{
							$params[$i] = str_replace('-', '_', $m2[0]) . '_id';
						}
					}
					if (isset($m[$i]) && $params[$i] != '')
					{
						if (substr($params[$i], -3) == '_id')
						{
							$_GET[$params[$i]] = (int) $m[$i];
						}
						else
						{
							$_GET[$params[$i]] = $m[$i];
						}
					}
				}
				for ($i = $s = $v = 1; $i < count($m); $i++)
				{
					if ($m[$i] == 'page' || (isset($m[$i - 1]) && $m[$i - 1] == 'page'))
					{
						continue;
					}
					$_GET['params'][] = $m[$i];
					if ($params[$i] == '')
					{
						$_GET['section_' . $s++] = $m[$i];
					}
					else
					{
						$_GET['value_' . $v++] = $m[$i];
					}
				}
				$_GET['q_pageless'] = preg_replace('`/page/\d+$`', '', $_GET['q']);
				$_GET['section'] = $q[0];
				return;
			}
		}

		// On supprime la requête si elle ne correspond à aucun modèle.
		unset($_GET['q']);
	}
}
?>