<?php
declare(strict_types = 1);

/**
 * Gestion de l'utilisateur connecté
 * (authentification, informations, session, sécurité).
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
class Auth
{
	/**
	 * L'utilisateur est-il connecté ?
	 *
	 * @var bool
	 */
	public static $connected = FALSE;

	/**
	 * Permissions de groupe.
	 *
	 * @var array
	 */
	public static $groupPerms = [];

	/**
	 * Identifiant. Par défaut : 2 (invité).
	 *
	 * @var int
	 */
	public static $id = 2;

	/**
	 * Informations.
	 *
	 * @var array
	 */
	public static $infos = [];

	/**
	 * L'utilisateur connecté est-il un administrateur ?
	 *
	 * @var bool
	 */
	public static $isAdmin = FALSE;

	/**
	 * Langue.
	 *
	 * @var string
	 */
	public static $lang = 'fr';

	/**
	 * Pseudonyme.
	 *
	 * @var string
	 */
	public static $nickname;

	/**
	 * Cookie des préférences.
	 *
	 * @var Cookie
	 */
	public static $prefs;

	/**
	 * Cookie de session.
	 *
	 * @var Cookie
	 */
	public static $session;

	/**
	 * Fuseau horaire.
	 *
	 * @var string
	 */
	public static $tz = 'Europe/Paris';



	/**
	 * Jeton anti-CSRF.
	 *
	 * @var string
	 */
	private static $_antiCSRFToken = '';

	/**
	 * Jeton de session.
	 *
	 * @var string
	 */
	private static $_sessionToken = NULL;



	/**
	 * Gestion du jeton anti-CSRF.
	 *
	 * La date d'expiration du jeton (CONF_ANTICSRF_TOKEN_EXPIRE) n'offre pas
	 * plus de sécurité, mais elle offre plus de confort pour l'utilisateur
	 * en lui permettant de travailler sur plusieurs onglets à la fois.
	 *
	 * En mettant CONF_ANTICSRF_TOKEN_EXPIRE à 0, la sécurité sera maximale,
	 * car le jeton sera recréé à chaque requête, mais du coup, on perd le
	 * confort décrit précédemment.
	 *
	 * @return bool
	 *   Retourne FALSE si le jeton n'est pas valide.
	 */
	public static function antiCSRFToken(): bool
	{
		// Si aucune donnée en POST, inutile de vérifier le jeton.
		if (!$_POST)
		{
			$valid = TRUE;
		}

		// Vérification du jeton.
		else if (isset($_POST['anticsrf']) && Utility::isSha1($_POST['anticsrf'])
		&& $_POST['anticsrf'] === self::$session->read('anticsrf'))
		{
			$valid = TRUE;
		}

		// Si le jeton n'est pas valide, on efface toutes les données en POST.
		else
		{
			$_FILES = [];
			$_POST = [];
			$valid = FALSE;
		}

		// Génère un nouveau jeton si le précédent a expiré.
		if (((int) self::$session->read('anticsrf_expire') - time()) < 1)
		{
			self::$_antiCSRFToken = Utility::keyHMAC();
			self::$session->add('anticsrf', self::$_antiCSRFToken);
		}
		else
		{
			self::$_antiCSRFToken = self::$session->read('anticsrf');
		}

		// Enregistre une nouvelle date d'expiration du jeton dans le cookie.
		self::$session->add('anticsrf_expire',
			(string) (time() + CONF_ANTICSRF_TOKEN_EXPIRE));

		return $valid;
	}

	/**
	 * Authentification par cookie et récupération
	 * des informations du compte utilisateur.
	 *
	 * @return bool
	 *   Retourne TRUE si l'authentification a réussie.
	 */
	public static function cookie(): bool
	{
		// Permissions de l'utilisateur "invité".
		$guest_perms = function()
		{
			if (Config::$params['users'])
			{
				if (DB::execute("SELECT group_perms FROM {groups} WHERE group_id = 2"))
				{
					self::$groupPerms = Utility::jsonDecode(DB::fetchVal());
				}
			}
		};

		// Récupération de l'identifiant de session que possède l'utilisateur.
		if (!$session_token = self::getCookieSessionToken())
		{
			$guest_perms();
			return FALSE;
		}

		// Récupération des informations utilisateur
		// si l'identifiant de session est valide.
		$sql = "SELECT g.*,
					   u.*
				  FROM {sessions} AS s,
				       {sessions_users} AS su,
					   {groups} AS g,
					   {users} AS u
				 WHERE u.user_id = su.user_id
				   AND su.session_id = s.session_id
				   AND u.group_id = g.group_id
				   AND user_status = '1'
				   AND session_token = ?
				   AND session_expire > NOW()";
		if (!DB::execute($sql, $session_token) || (!self::$infos = DB::fetchRow())
		|| (!Config::$params['users'] && self::$infos['group_admin'] != 1))
		{
			$guest_perms();
			return FALSE;
		}

		// Décodage des tableaux.
		self::$infos['user_prefs'] = Utility::jsonDecode(self::$infos['user_prefs']);
		if (!is_array(self::$infos['user_prefs']))
		{
			self::$infos['user_prefs'] = [];
		}

		// Permissions de groupe.
		self::$isAdmin = self::$infos['group_admin'] == 1;
		self::$groupPerms = Utility::jsonDecode(self::$infos['group_perms']);
		unset(self::$infos['group_perms']);

		// Utilisateur non authentifié.
		if (self::$infos['user_id'] == 2)
		{
			return FALSE;
		}

		// Un administrateur a toujours tous les droits.
		if (self::$isAdmin)
		{
			self::$groupPerms = Config::getDBParamDefault('users_group_perms_default');
			self::$groupPerms['upload'] = 1;
		}

		// Seul un administrateur peut accéder à la partie d'administration.
		if (App::$scriptName == 'admin' && !self::$isAdmin)
		{
			return FALSE;
		}

		// Seul l'utilisateur avec l'identifiant 1 peut être superadmin.
		if (self::$infos['user_id'] != 1 && self::$infos['group_id'] == 1)
		{
			return FALSE;
		}
		self::$id = (int) self::$infos['user_id'];

		// Langue et fuseau horaire.
		self::$lang = self::$infos['user_lang'];
		self::$tz = self::$infos['user_tz'];

		// Pseudonyme.
		self::$nickname = User::getNickname(
			self::$infos['user_login'], self::$infos['user_nickname']
		);

		// Mise à jour de la date de dernière visite.
		$sql = "UPDATE {users}
				   SET user_lastvstdt = NOW(),
					   user_lastvstip = ?
				 WHERE user_id = ?";
		DB::execute($sql, [$_SERVER['REMOTE_ADDR'], self::$infos['user_id']]);

		// Mise à jour de la date d'expiration de la session.
		$session_expire = (int) CONF_SESSION_EXPIRE;
		$sql = "UPDATE {sessions}
				   SET session_expire = DATE_ADD(NOW(), INTERVAL $session_expire SECOND)
				 WHERE session_token = ?";
		DB::execute($sql, $session_token);

		return self::$connected = TRUE;
	}

	/**
	 * Authentification par formulaire.
	 *
	 * @param string $login
	 *   Identifiant de connexion.
	 * @param string $password
	 *   Mot de passe.
	 * @param bool $remember
	 *   Option pour se "souvenir" de l'utilisateur.
	 *
	 * @return bool
	 *   Retourne TRUE si l'authentification a réussie,
	 *   FALSE sinon.
	 */
	public static function form(string $login, string $password, bool $remember = FALSE): bool
	{
		if (self::$connected)
		{
			return FALSE;
		}

		// Filtrage des données.
		if (strlen($password) > User::COLUMNS_LENGTH_MAX['password']
		|| strlen($login) > User::COLUMNS_LENGTH_MAX['login']
		|| !preg_match('`^' . User::LOGIN_PATTERN . '$`i', $login))
		{
			return FALSE;
		}

		// Vérification par liste noire des adresses IP.
		if (App::blacklists('login_' . App::$scriptName,
		'', '', '', ['login' => $login]) !== TRUE)
		{
			return FALSE;
		}

		// Prévention contre les attaques par force brute.
		if (self::isBruteForce('login\_%\_rejected'))
		{
			die('Brute force attack detected.');
		}

		// Récupération des informations utiles de l'utilisateur.
		$sql = "SELECT user_id,
					   user_crtdt,
					   user_password,
					   user_lang,
					   user_tz,
					   group_admin
				  FROM {users}
			 LEFT JOIN {groups} USING (group_id)
				 WHERE group_id != 2
				   AND user_status = '1'
				   AND user_login = ?
				   AND user_id != 2";
		if (!DB::execute($sql, $login))
		{
			return FALSE;
		}
		if (empty(($user_infos = DB::fetchRow())['user_id']))
		{
			goto fail;
		}

		// Seul un administrateur peut accéder à la partie d'administration.
		if (App::$scriptName == 'admin' && $user_infos['group_admin'] != 1)
		{
			goto fail;
		}

		// Vérification du mot de passe.
		if (!App::passwordVerify($password, $user_infos['user_password']))
		{
			goto fail;
		}

		// Début de la transaction.
		$in_transaction = DB::inTransaction();
		if (!$in_transaction && !DB::beginTransaction())
		{
			return FALSE;
		}

		self::$id = (int) $user_infos['user_id'];

		// Nouvel identifiant de session.
		$session = self::getNewSessionToken();
		if (!$session['id'])
		{
			return FALSE;
		}

		// On associe l'identifiant de session avec l'utilisateur.
		$sql = "INSERT INTO {sessions_users} (session_id, user_id) VALUES (?, ?)";
		if (!DB::execute($sql, [$session['id'], self::$id]))
		{
			return FALSE;
		}

		// Mise à jour du cookie de session.
		self::$session->add('session_token', $session['token']);

		// Log d'activité.
		App::logActivity('login_' . App::$scriptName, '', ['login' => $login]);

		// Exécution de la transaction.
		if (!$in_transaction && !DB::commitTransaction())
		{
			return FALSE;
		}

		// Expiration du cookie de session.
		self::$session->add(
			'session_expire', $remember ? (string) CONF_COOKIE_SESSION_EXPIRE : '0'
		);

		return TRUE;


		fail:

		// Log d'activité.
		App::logActivity('login_' . App::$scriptName . '_rejected', '', ['login' => $login]);

		return FALSE;
	}

	/**
	 * Retourne le jeton anti-CSRF.
	 *
	 * @return string
	 */
	public static function getAntiCSRFToken(): string
	{
		return self::$_antiCSRFToken;
	}

	/**
	 * Retourne l'identifiant de session que possède l'utilisateur.
	 *
	 * @return string
	 */
	public static function getCookieSessionToken(): string
	{
		if (self::$_sessionToken === NULL)
		{
			$session_token = self::$session->read('session_token');
			self::$_sessionToken = Utility::isSha1($session_token) ? $session_token : '';
		}

		return self::$_sessionToken;
	}

	/**
	 * Génère un nouvel identifiant de session et
	 * retourne les informations utiles de la session.
	 *
	 * @return array
	 */
	public static function getNewSessionToken(): array
	{
		$session_id = 0;
		$session_token = self::getCookieSessionToken();
		$session_valid = FALSE;

		// L'utilisateur possède-t-il déjà un jeton de session valide ?
		if ($session_token)
		{
			$sql = "SELECT session_id
					  FROM {sessions}
					 WHERE session_token = ?
					   AND session_expire > NOW()";
			if (!DB::execute($sql, $session_token))
			{
				goto end;
			}
			if (isset(($i = DB::fetchRow())['session_id']))
			{
				$session_valid = TRUE;
				$session_id = $i['session_id'];
			}
		}

		// Nouveau jeton de session.
		$new_session_token = Utility::keyHMAC();

		// Si l'utilisateur possède un jeton de session valide,
		// on remplace l'actuel par le nouveau.
		if ($session_valid)
		{
			$sql = "UPDATE {sessions} SET session_token = ? WHERE session_token = ?";
			$params = [$new_session_token, $session_token];
			if (!DB::execute($sql, $params) || DB::rowCount() != 1)
			{
				goto end;
			}
		}

		// Sinon, on insère le nouveau dans la base de données.
		else
		{
			$session_expire = (int) CONF_SESSION_EXPIRE;
			$sql = "INSERT INTO {sessions} (session_token, session_expire)
						 VALUES (?, DATE_ADD(NOW(), INTERVAL $session_expire SECOND))";
			$params = [$new_session_token];
			$seq = ['table' => '{sessions}', 'column' => 'session_id'];
			if (!DB::execute($sql, $params, $seq) || DB::rowCount() != 1)
			{
				goto end;
			}
			$session_id = DB::lastInsertId();
		}

		$session_token = $new_session_token;

		end:
		return
		[
			'id' => $session_id,
			'token' => $session_token
		];
	}

	/**
	 * Détermine si une attaque par force brute a été détectée,
	 * en limitant le nombre d'action $action par jour depuis une même IP.
	 *
	 * @param string $action
	 *
	 * @return bool
	 */
	public static function isBruteForce(string $action): bool
	{
		$sql = "SELECT COUNT(*)
				  FROM {users_logs}
				 WHERE log_ip = ?
				   AND log_action LIKE '$action'
				   AND (TO_SECONDS(NOW()) - TO_SECONDS(log_date)) < 86400";
		if (!DB::execute($sql, $_SERVER['REMOTE_ADDR'])
		  || DB::fetchVal() > CONF_BRUTE_FORCE_MAX_ATTEMPT)
		{
			return TRUE;
		}

		return FALSE;
	}

	/**
	 * Déconnexion de l'utilisateur.
	 *
	 * @return bool
	 */
	public static function logout(): bool
	{
		if (!self::$connected)
		{
			return FALSE;
		}

		// Démarre une transaction.
		if (!DB::beginTransaction())
		{
			return FALSE;
		}

		// Suppression de l'association entre l'identifiant de session et l'utilisateur.
		$sql = "SELECT session_id
				  FROM {sessions}
				 WHERE session_token = ?
				   AND session_expire > NOW()";
		$sql = "DELETE
				  FROM {sessions_users}
				 WHERE session_id = ($sql)
				   AND user_id = ?";
		$params = [self::getCookieSessionToken(), self::$id];
		if (!DB::execute($sql, $params))
		{
			return FALSE;
		}

		// Log d'activité.
		App::logActivity('logout_' . App::$scriptName);

		// On valide la transaction.
		if (!DB::commitTransaction())
		{
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Gestion des cookies.
	 *
	 * @return void
	 */
	public static function sessionHandler()
	{
		// Initialisation des cookies.
		self::$prefs = new Cookie(CONF_COOKIE_PREFS_NAME);
		self::$session = new Cookie(CONF_COOKIE_SESSION_NAME);

		// Écriture des cookies.
		header_register_callback(function()
		{
			if (is_object(self::$prefs))
			{
				self::$prefs->write((int) CONF_COOKIE_PREFS_EXPIRE);
				self::$prefs = NULL;
			}
			if (is_object(self::$session))
			{
				self::$session->write((int) self::$session->read('session_expire'));
				self::$session = NULL;
			}
		});
	}
}
?>