<?php
declare(strict_types = 1);

/**
 * Traitement des requêtes Ajax pour la galerie.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
class AjaxGallery extends Ajax
{
	/**
	 * Vérification du mot de passe de catégorie.
	 *
	 * @return void
	 */
	public static function checkCatPassword(): void
	{
		require_once(__DIR__ . '/GalleryCategory.class.php');

		$r = GalleryCategory::checkPassword('category', (int) $_POST['cat_id'],
			$_POST['password'] ?? '', !empty($_POST['remember']));

		if ($r == 'db_error')
		{
			self::_dbError();
		}
		if ($r == 'not_found')
		{
			self::_forbidden();
		}
		if ($r == 'bruteforce')
		{
			self::_printResponse(
			[
				'status' => 'error',
				'message' => 'Brute force attack detected.'
			]);
		}
		if (substr($r, 0, 9) == 'redirect:')
		{
			self::_printResponse(
			[
				'status' => 'success',
				'url' => App::getURL(substr($r, 9))
			]);
		}

		self::_printResponse(
		[
			'status' => 'error',
			'message' => __('Mot de passe incorrect.')
		]);
	}

	/**
	 * Supprime un commentaire.
	 *
	 * @return void
	 */
	public static function deleteComment(): void
	{
		// Permissions d'accès au commentaire.
		$com_id = self::_commentPermission();

		// Suppression du commentaire.
		$r = Comment::delete([$com_id]);

		// Message de rapport.
		if ($r == -1)
		{
			self::_printResponse(
			[
				'status' => 'error',
				'message' => Report::getErrorDefaultMessage()
			]);
		}
		else if ($r > 0)
		{
			// Log d'activité.
			App::logActivity('comment_delete', '', ['com_id' => $com_id]);

			self::_printResponse(
			[
				'status' => 'success',
				'message' => __('Le commentaire a été supprimé.')
			]);
		}
	}

	/**
	 * Supprime un fichier.
	 *
	 * @return void
	 */
	public static function deleteItem(): void
	{
		// Vérifications.
		if (!isset($_POST['item_id']) || !preg_match('`^\d{1,12}$`', $_POST['item_id']))
		{
			self::_forbidden();
		}
		$id = (int) $_POST['item_id'];

		// Permissions d'accès au fichier.
		$item_infos = self::_itemPermission($id);

		// Permissions de suppression du fichier.
		if ((Auth::$isAdmin || (Config::$params['users']
		&& $item_infos['user_id'] == Auth::$id)) === FALSE)
		{
			self::_forbidden();
		}

		$r = Item::delete([$id]);

		// Message de rapport.
		if ($r == -1)
		{
			self::_printResponse(
			[
				'status' => 'error',
				'message' => Report::getErrorDefaultMessage()
			]);
		}
		else if ($r > 0)
		{
			// Log d'activité.
			App::logActivity('item_delete', '', ['item_id' => $_POST['item_id']]);

			self::_printResponse(
			[
				'status' => 'success',
				'message' => __('Le fichier a été supprimé.'),
				'redirect' => CONF_GALLERY_PATH . '/'
			]);
		}
	}

	/**
	 * Édition des informations d'une catégorie :
	 * titre et description.
	 *
	 * @return void
	 */
	public static function editCategory(): void
	{
		// Vérifications.
		if (!isset($_POST['cat_id']) || !preg_match('`^\d{1,12}$`', $_POST['cat_id']))
		{
			self::_forbidden();
		}
		$id = (int) $_POST['cat_id'];

		// Permissions d'accès à la catégorie.
		$cat_infos = self::_catPermission($id);

		// Permissions de modification de la catégorie.
		if ($cat_infos['cat_id'] > 1 && (Auth::$isAdmin || (Config::$params['users']
		&& Auth::$id != 2 && $cat_infos['user_id'] == Auth::$id)) === FALSE)
		{
			self::_forbidden();
		}

		// Préparation des données.
		$data = [];
		if (isset($_POST['title']))
		{
			$data['cat_name'] = $data['cat_url'] = $_POST['title'];
		}
		if (isset($_POST['description']))
		{
			$data['cat_desc'] = $_POST['description'];
		}
		if (!$data)
		{
			self::_forbidden();
		}

		// Mise à jour de la base de données.
		$data = [$id => $data];
		$r = Category::edit($data);

		// Erreur.
		if ($r == -1)
		{
			self::_printResponse(
			[
				'status' => 'error',
				'message' => Report::getErrorDefaultMessage()
			]);
			return;
		}

		// Récupération des informations en base de données.
		if (!DB::execute("SELECT * FROM {categories} WHERE cat_id = ?", $id))
		{
			self::_dbError();
		}
		$i = DB::fetchRow();

		// Formatage des données.
		$params = [];
		if (isset($_POST['title']))
		{
			$params['title'] = HTML::specialchars($i['cat_name']);
		}
		if (isset($_POST['description']))
		{
			$desc = HTML::specialchars($i['cat_desc']);
			$params['description_formated'] = $params['description'] = NULL;
			if ($i['cat_desc'] !== NULL)
			{
				$params['description'] = $desc;
				$params['description_formated'] = Template::formatDescription($desc);
			}
		}

		// Modification réussie.
		if ($r > 0)
		{
			// Log d'activité.
			App::logActivity('cat_edit', '', ['cat_id' => $_POST['cat_id']]);

			$params['status'] = 'success';
			$params['message'] = Report::getSuccessDefaultMessage();
		}

		// Aucun changement.
		else
		{
			$params['status'] = 'info';
			$params['message'] = Report::getNoChangeDefaultMessage();
		}

		self::_printResponse($params);
	}

	/**
	 * Édition d'un commentaire.
	 *
	 * @return void
	 */
	public static function editComment(): void
	{
		if (!isset($_POST['com_message']))
		{
			self::_forbidden();
		}

		// Permissions d'accès au commentaire.
		$com_id = self::_commentPermission();

		// Vérification du message : format.
		if (($r = Comment::checkMessage($_POST['com_message'], FALSE)) !== NULL)
		{
			self::_printResponse(
			[
				'status' => 'error',
				'message' => $r['message']
			]);
		}

		// Vérification du message : listes noires.
		if (is_array($r = App::blacklists('', '', '', $_POST['com_message'])))
		{
			self::_printResponse(
			[
				'status' => 'error',
				'message' => $r['text']
			]);
		}

		// Vérification du message : suppression des émojis ?
		if (Config::$params['comments_emoji_delete'])
		{
			$_POST['com_message'] = preg_replace(
				'`' . Emoji::REGEXP . '`u', '', $_POST['com_message']
			);
		}

		// Vérification du message : message vide ?
		if (Utility::isEmpty($_POST['com_message']))
		{
			self::_printResponse(
			[
				'status' => 'info',
				'message' => __('Le message que vous avez envoyé est vide.')
			]);
		}

		// Champs éditables.
		$fields = ['com_author', 'com_email', 'com_message', 'com_website'];

		// Préparation des données.
		$data = [];
		foreach ($fields as $f)
		{
			if (isset($_POST[$f]))
			{
				$data[substr($f, 4)] = $_POST[$f];
			}
		}
		if (!$data)
		{
			self::_forbidden();
		}

		// Mise à jour de la base de données.
		$data = [$com_id => $data];
		$r = Comment::edit($data);

		// Erreur.
		if ($r == -1)
		{
			self::_printResponse(
			[
				'status' => 'error',
				'message' => Report::getErrorDefaultMessage()
			]);
			return;
		}

		// Récupération des informations en base de données.
		if (!DB::execute("SELECT * FROM {comments} WHERE com_id = ?", $com_id))
		{
			self::_dbError();
		}
		$i = DB::fetchRow();

		// Formatage des données.
		$params = [];
		foreach ($fields as $f)
		{
			if (isset($_POST[$f]))
			{
				$params[$f] = HTML::specialchars($i[$f]);
				if ($f == 'com_message')
				{
					$params['com_message_formated'] = Template::formatComment($params[$f]);
				}
			}
		}

		// Modification réussie.
		if ($r > 0)
		{
			// Log d'activité.
			App::logActivity('comment_edit', '', ['com_id' => $com_id]);

			$params['status'] = 'success';
			$params['message'] = Report::getSuccessDefaultMessage();
		}

		// Aucun changement.
		else
		{
			$params['status'] = 'info';
			$params['message'] = Report::getNoChangeDefaultMessage();
		}

		self::_printResponse($params);
	}

	/**
	 * Édition des informations d'un fichier :
	 * titre, nom de fichier, description et tags.
	 *
	 * @return void
	 */
	public static function editItem(): void
	{
		// Vérifications.
		if (!isset($_POST['item_id']) || !preg_match('`^\d{1,12}$`', $_POST['item_id']))
		{
			self::_forbidden();
		}
		$id = (int) $_POST['item_id'];

		// Permissions d'accès au fichier.
		$item_infos = self::_itemPermission($id);

		// Permissions de modification du fichier.
		if ((Auth::$isAdmin || (Config::$params['users']
		&& Auth::$id != 2 && $item_infos['user_id'] == Auth::$id)) === FALSE)
		{
			self::_forbidden();
		}

		// Préparation des données.
		$data = [];
		if (isset($_POST['title']))
		{
			$data['item_name'] = $data['item_url'] = $_POST['title'];
		}
		if (isset($_POST['filename']))
		{
			$data['item_path'] = $_POST['filename'];
		}
		if (isset($_POST['description']))
		{
			$data['item_desc'] = $_POST['description'];
		}
		if (Config::$params['tags'] && isset($_POST['tags']))
		{
			$data['tags'] = $_POST['tags'];
		}
		if (!$data)
		{
			self::_forbidden();
		}

		// Mise à jour de la base de données.
		$data = [$id => $data];
		$r = Item::edit($data);

		// Erreur.
		if ($r == -1)
		{
			self::_printResponse(
			[
				'status' => 'error',
				'message' => Report::getErrorDefaultMessage()
			]);
			return;
		}

		// Récupération des informations en base de données.
		if (!DB::execute("SELECT * FROM {items} WHERE item_id = ?", $id))
		{
			self::_dbError();
		}
		$i = DB::fetchRow();

		// Formatage des données.
		$params = [];
		if (isset($_POST['title']))
		{
			$params['title'] = HTML::specialchars($i['item_name']);
		}
		if (isset($_POST['filename']))
		{
			$filename = basename($i['item_path']);
			$params['download'] = App::getFileSource($i['item_path'], TRUE);
			$params['filename'] = HTML::specialchars($filename);
			$params['is_video'] = Item::isVideo($i['item_type']);
			$params['source'] = Item::isImage($i['item_type'])
				&& (Config::$params['items_resize']
				|| (Config::$params['users'] && !Auth::$groupPerms['image_original']))
				? App::getImageResize($i)
				: App::getFileSource($i['item_path']);
		}
		if (isset($_POST['description']))
		{
			$desc = HTML::specialchars($i['item_desc']);
			$params['description_formated'] = $params['description'] = NULL;
			if ($i['item_desc'] !== NULL)
			{
				$params['description'] = $desc;
				$params['description_formated'] = Template::formatDescription($desc);
			}
		}
		if (isset($_POST['tags']))
		{
			require_once(__DIR__ . '/GalleryItems.class.php');
			$tags = GalleryItems::getTags($i);
			$tags_list = [];
			foreach ($tags as &$i)
			{
				$tags_list[] = $i['name'];
			}
			$params['tags_list'] = $tags_list;
			$params['tags'] = HTML::specialchars($tags);
		}

		// Modification réussie.
		if ($r > 0)
		{
			// Log d'activité.
			App::logActivity('item_edit', '', ['item_id' => $_POST['item_id']]);

			$params['status'] = 'success';
			$params['message'] = Report::getSuccessDefaultMessage();
		}

		// Aucun changement.
		else
		{
			$params['status'] = 'info';
			$params['message'] = Report::getNoChangeDefaultMessage();
		}

		self::_printResponse($params);
	}

	/**
	 * Ajoute ou retire un fichier des favoris d'un utilisateur.
	 *
	 * @param string $action
	 *   Action à effectuer : 'add' ou 'remove'.
	 *
	 * @return void
	 */
	public static function favorites(string $action): void
	{
		// Quelques vérifications.
		if (!Config::$params['users'] || !isset($_POST['item_id'])
		|| !preg_match('`^\d{1,12}$`', $_POST['item_id']))
		{
			self::_forbidden();
		}

		// Permissions d'accès au fichier.
		self::_itemPermission((int) $_POST['item_id']);

		// On modifie les favoris.
		$r = Item::favorites([$_POST['item_id']], $action, Auth::$id);
		if ($r == 0)
		{
			self::_forbidden();
		}
		if ($r == -1)
		{
			self::_dbError();
		}

		// Log d'activité.
		App::logActivity('favorites_' . $action, '', ['item_id' => $_POST['item_id']]);

		// Récupération du nombre de favoris du fichier.
		$sql = "SELECT item_favorites FROM {items} WHERE item_id = ?";
		$item_favorites = (int) (DB::execute($sql, $_POST['item_id']) ? DB::fetchVal() : 0);

		self::_printResponse([
			'favorites' => $item_favorites,
			'favorites_short' => L10N::formatShortNumber($item_favorites),
			'status' => 'success'
		]);
	}

	/**
	 * Connexion utilisateur.
	 *
	 * @return void
	 */
	public static function login(): void
	{
		if (Auth::form($_POST['login'], $_POST['password'], !empty($_POST['remember'])))
		{
			self::_printResponse(
			[
				'status' => 'success'
			]);
		}
		else
		{
			self::_printResponse(
			[
				'status' => 'error',
				'message' => __('Informations incorrectes.')
			]);
		}
	}

	/**
	 * Enregistrement des paramètres de thème.
	 *
	 * @return void
	 */
	public static function themeParams(): void
	{
		// Vérifications.
		if (!Auth::$isAdmin || !isset($_POST['params']) || !is_array($_POST['params'])
		|| !isset($_POST['template']) || !preg_match('`^[a-z0-9_]{1,50}$`i', $_POST['template'])
		|| $_POST['template'] != Config::$params['theme_template'])
		{
			self::_forbidden();
		}

		if (mb_strlen(Utility::jsonEncode($_POST['params'])) > 10000)
		{
			self::_printResponse(
			[
				'status' => 'error',
				'message' => 'Array too large.'
			]);
		}

		// Enregistrement.
		$params = is_array(Config::$params['theme_params'])
			? Config::$params['theme_params']
			: [];
		$params[$_POST['template']] = $_POST['params'];
		$r = Config::changeDBParams(['theme_params' => $params]);

		// Message de rapport.
		if ($r == -1)
		{
			self::_printResponse(
			[
				'status' => 'error',
				'message' => Report::getErrorDefaultMessage()
			]);
		}
		else if ($r > 0)
		{
			self::_printResponse(
			[
				'status' => 'success',
				'message' => Report::getSuccessDefaultMessage()
			]);
		}
		else
		{
			self::_printResponse(
			[
				'status' => 'info',
				'message' => Report::getNoChangeDefaultMessage()
			]);
		}
	}

	/**
	 * Récupération de l'URL de vignettes de fichiers.
	 *
	 * @return void
	 */
	public static function thumbsItemsSource(): void
	{
		// Vérifications.
		if (!Auth::$isAdmin || !isset($_POST['items_id']) || !is_array($_POST['items_id'])
		|| !isset($_POST['size']) || !preg_match('`^\d{1,4}(x\d{1,4})?$`i', $_POST['size']))
		{
			self::_forbidden();
		}

		$items_id = DB::inInt($_POST['items_id']);
		if (!DB::execute("SELECT * FROM {items} WHERE item_id IN ($items_id)"))
		{
			self::_dbError();
		}

		$thumbs_source = [];
		foreach (DB::fetchAll() as &$i)
		{
			$thumbs_source[$i['item_id']] = App::getThumbSource('item', $i, $_POST['size'], '');
		}

		self::_printResponse(
		[
			'status' => 'success',
			'thumbs_source' => $thumbs_source
		]);
	}



	/**
	 * Vérifie les permissions d'accès à une catégorie.
	 *
	 * @param int $cat_id
	 *
	 * @return array
	 */
	private static function _catPermission(int $cat_id): array
	{
		$sql = "SELECT *
				  FROM {categories} AS cat
				 WHERE cat_id = ?
				   AND cat_status = '1'
				   AND %s
				   AND %s";
		$sql = sprintf($sql, SQL::catPerms(), SQL::catPassword());
		if (!DB::execute($sql, $cat_id))
		{
			self::_dbError();
		}
		if (!$infos = DB::fetchRow())
		{
			self::_forbidden();
		}

		return $infos;
	}

	/**
	 * Vérifie les permissions d'accès à un commentaire.
	 *
	 * @return int
	 */
	private static function _commentPermission(): int
	{
		// Vérifications.
		if (!isset($_POST['com_id']) || !preg_match('`^\d{1,12}$`', $_POST['com_id']))
		{
			self::_forbidden();
		}
		$com_id = (int) $_POST['com_id'];

		// Permission d'édition du commentaire.
		$sql = "SELECT i.item_id,
					   com.user_id
				  FROM {comments} AS com
			 LEFT JOIN {items} AS i USING (item_id)
				 WHERE com_id = ?
				   AND com_status = '1'";
		if (!DB::execute($sql, $com_id))
		{
			self::_dbError();
		}
		if (!$com_infos = DB::fetchRow())
		{
			self::_forbidden();
		}
		if ((Auth::$isAdmin || (Config::$params['users']
		&& Auth::$id != 2 && $com_infos['user_id'] == Auth::$id)) === FALSE)
		{
			self::_forbidden();
		}

		// Permissions d'accès au fichier.
		self::_itemPermission((int) $com_infos['item_id']);

		return $com_id;
	}

	/**
	 * Vérifie les permissions d'accès à un fichier.
	 *
	 * @param int $item_id
	 *
	 * @return array
	 */
	private static function _itemPermission(int $item_id): array
	{
		$sql = "SELECT i.*
				  FROM {items} AS i
			 LEFT JOIN {categories} AS cat USING (cat_id)
				 WHERE item_id = ?
				   AND item_status = '1'
				   AND %s
				   AND %s";
		$sql = sprintf($sql, SQL::catPerms(), SQL::catPassword());
		if (!DB::execute($sql, $item_id))
		{
			self::_dbError();
		}
		if (!$infos = DB::fetchRow())
		{
			self::_forbidden();
		}

		return $infos;
	}
}
?>